@@ -2,7 +2,7 @@
 #
 #       codeboxes.py
 #
-#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2017 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 
 import gtk, gtksourceview2, pango
 import os
-import cons, menus, support
+import cons, menus, support, config
 
 DRAW_SPACES_FLAGS = gtksourceview2.DRAW_SPACES_ALL & ~gtksourceview2.DRAW_SPACES_NEWLINE
 CB_WIDTH_HEIGHT_STEP_PIX = 15
@@ -41,14 +41,14 @@ class CodeBoxesHandler:
         self.curr_h = 0
         self.key_down = False
 
-    def codebox_in_use(self):
+    def codebox_in_use_get_anchor(self):
         """Returns a CodeBox SourceView if Currently in Use or None"""
         if not self.curr_codebox_anchor: return None
         if not self.dad.curr_buffer: return None
         iter_sel_start = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
         anchor = iter_sel_start.get_child_anchor()
         if not anchor: return None
-        if "sourcebuffer" in dir(anchor): return anchor.sourceview
+        if "sourcebuffer" in dir(anchor): return anchor
         return None
 
     def codebox_cut(self, *args):
@@ -87,25 +87,23 @@ class CodeBoxesHandler:
         dialog.set_default_size(300, -1)
         dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
 
-        combobox_prog_lang = gtk.ComboBox(model=self.dad.prog_lang_liststore)
-        cell = gtk.CellRendererText()
-        combobox_prog_lang.pack_start(cell, True)
-        combobox_prog_lang.add_attribute(cell, 'text', 0)
-        combobox_value = self.dad.codebox_syn_highl if self.dad.codebox_syn_highl != cons.PLAIN_TEXT_ID else self.dad.auto_syn_highl
-        combobox_iter = self.dad.get_combobox_iter_from_value(self.dad.prog_lang_liststore, 1, combobox_value)
-        combobox_prog_lang.set_active_iter(combobox_iter)
+        button_prog_lang = gtk.Button()
+        button_label = self.dad.codebox_syn_highl if self.dad.codebox_syn_highl != cons.PLAIN_TEXT_ID else self.dad.auto_syn_highl
+        button_stock_id = config.get_stock_id_for_code_type(button_label)
+        button_prog_lang.set_label(button_label)
+        button_prog_lang.set_image(gtk.image_new_from_stock(button_stock_id, gtk.ICON_SIZE_MENU))
         radiobutton_plain_text = gtk.RadioButton(label=_("Plain Text"))
         radiobutton_auto_syntax_highl = gtk.RadioButton(label=_("Automatic Syntax Highlighting"))
         radiobutton_auto_syntax_highl.set_group(radiobutton_plain_text)
         if self.dad.codebox_syn_highl == cons.PLAIN_TEXT_ID:
             radiobutton_plain_text.set_active(True)
-            combobox_prog_lang.set_sensitive(False)
+            button_prog_lang.set_sensitive(False)
         else:
             radiobutton_auto_syntax_highl.set_active(True)
         type_vbox = gtk.VBox()
         type_vbox.pack_start(radiobutton_plain_text)
         type_vbox.pack_start(radiobutton_auto_syntax_highl)
-        type_vbox.pack_start(combobox_prog_lang)
+        type_vbox.pack_start(button_prog_lang)
         type_frame = gtk.Frame(label="<b>"+_("Type")+"</b>")
         type_frame.get_label_widget().set_use_markup(True)
         type_frame.set_shadow_type(gtk.SHADOW_NONE)
@@ -172,8 +170,18 @@ class CodeBoxesHandler:
         content_area.pack_start(size_frame)
         content_area.pack_start(options_frame)
         content_area.show_all()
+        def on_button_prog_lang_clicked(button):
+            icon_n_key_list = []
+            for key in self.dad.available_languages:
+                stock_id = config.get_stock_id_for_code_type(key)
+                icon_n_key_list.append([key, stock_id, key])
+            sel_key = support.dialog_choose_element_in_list(self.dad.window, _("Automatic Syntax Highlighting"), [], "", icon_n_key_list)
+            if sel_key:
+                button.set_label(sel_key)
+                button.set_image(gtk.image_new_from_stock(sel_key, gtk.ICON_SIZE_MENU))
+        button_prog_lang.connect('clicked', on_button_prog_lang_clicked)
         def on_radiobutton_auto_syntax_highl_toggled(radiobutton):
-            combobox_prog_lang.set_sensitive(radiobutton.get_active())
+            button_prog_lang.set_sensitive(radiobutton.get_active())
         def on_key_press_codeboxhandle(widget, event):
             keyname = gtk.gdk.keyval_name(event.keyval)
             if keyname == cons.STR_KEY_RETURN:
@@ -203,7 +211,7 @@ class CodeBoxesHandler:
             if radiobutton_plain_text.get_active():
                 self.dad.codebox_syn_highl = cons.PLAIN_TEXT_ID
             else:
-                self.dad.codebox_syn_highl = self.dad.prog_lang_liststore[combobox_prog_lang.get_active_iter()][1]
+                self.dad.codebox_syn_highl = button_prog_lang.get_label()
             return True
         return False
 
@@ -296,7 +304,7 @@ class CodeBoxesHandler:
 
     def codebox_increase_width(self, *args):
         """Increase CodeBox Width"""
-        if self.dad.is_curr_node_read_only(): return
+        if self.dad.get_node_read_only(): return
         if self.curr_codebox_anchor.width_in_pixels:
             self.codebox_change_width_height(self.curr_codebox_anchor.frame_width + CB_WIDTH_HEIGHT_STEP_PIX, 0)
         else:
@@ -304,7 +312,7 @@ class CodeBoxesHandler:
 
     def codebox_decrease_width(self, *args):
         """Decrease CodeBox Width"""
-        if self.dad.is_curr_node_read_only(): return
+        if self.dad.get_node_read_only(): return
         if self.curr_codebox_anchor.width_in_pixels:
             if self.curr_codebox_anchor.frame_width - CB_WIDTH_HEIGHT_STEP_PIX >= CB_WIDTH_LIMIT_MIN:
                 self.codebox_change_width_height(self.curr_codebox_anchor.frame_width - CB_WIDTH_HEIGHT_STEP_PIX, 0)
@@ -314,12 +322,12 @@ class CodeBoxesHandler:
 
     def codebox_increase_height(self, *args):
         """Increase CodeBox Height"""
-        if self.dad.is_curr_node_read_only(): return
+        if self.dad.get_node_read_only(): return
         self.codebox_change_width_height(0, self.curr_codebox_anchor.frame_height + CB_WIDTH_HEIGHT_STEP_PIX)
 
     def codebox_decrease_height(self, *args):
         """Decrease CodeBox Height"""
-        if self.dad.is_curr_node_read_only(): return
+        if self.dad.get_node_read_only(): return
         if self.curr_codebox_anchor.frame_height - CB_WIDTH_HEIGHT_STEP_PIX >= CB_HEIGHT_LIMIT_MIN:
             self.codebox_change_width_height(0, self.curr_codebox_anchor.frame_height - CB_WIDTH_HEIGHT_STEP_PIX)
 
@@ -348,18 +356,18 @@ class CodeBoxesHandler:
     def codebox_load_from_file(self, action):
         """Load the CodeBox Content From a Text File"""
         if not self.dad.is_curr_node_not_read_only_or_error(): return
-        filepath = support.dialog_file_select(curr_folder=self.dad.pick_dir, parent=self.dad.window)
+        filepath = support.dialog_file_select(curr_folder=self.dad.pick_dir_cbox, parent=self.dad.window)
         if not filepath: return
-        self.dad.pick_dir = os.path.dirname(filepath)
+        self.dad.pick_dir_cbox = os.path.dirname(filepath)
         with open(filepath, 'r') as fd:
             self.curr_codebox_anchor.sourcebuffer.set_text(fd.read())
 
     def codebox_save_to_file(self, action):
         """Save the CodeBox Content To a Text File"""
-        filepath = support.dialog_file_save_as(curr_folder=self.dad.pick_dir,
+        filepath = support.dialog_file_save_as(curr_folder=self.dad.pick_dir_cbox,
                                                parent=self.dad.window)
         if not filepath: return
-        self.dad.pick_dir = os.path.dirname(filepath)
+        self.dad.pick_dir_cbox = os.path.dirname(filepath)
         with open(filepath, 'w') as fd:
             fd.write(self.curr_codebox_anchor.sourcebuffer.get_text(*self.curr_codebox_anchor.sourcebuffer.get_bounds()))
 
@@ -449,6 +457,11 @@ class CodeBoxesHandler:
                 if event.state & gtk.gdk.MOD1_MASK:
                     self.codebox_decrease_height()
                 else: self.codebox_increase_height()
+            elif keyname == "space":
+                text_iter = self.dad.curr_buffer.get_iter_at_child_anchor(self.curr_codebox_anchor)
+                text_iter.forward_char()
+                self.dad.curr_buffer.place_cursor(text_iter)
+                self.dad.sourceview.grab_focus()
         elif (event.state & gtk.gdk.SHIFT_MASK):
             if keyname == cons.STR_KEY_SHIFT_TAB:
                 text_buffer = widget.get_buffer()
@@ -484,7 +497,7 @@ class CodeBoxesHandler:
     def on_sourceview_motion_notify_event_codebox(self, text_view, event):
         """Update the cursor image if the pointer moved"""
         if not self.dad.user_active: return False
-        text_view.set_editable(not self.dad.is_curr_node_read_only())
+        text_view.set_editable(not self.dad.get_node_read_only())
         x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, int(event.x), int(event.y))
         support.sourceview_cursor_and_tooltips_handler(self.dad, text_view, x, y)
         return False
