@@ -2,7 +2,7 @@
 #
 #       cons.py
 #
-#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2017 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -19,10 +19,11 @@
 #       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #       MA 02110-1301, USA.
 
-import os, sys
+import os
+import sys
 
 
-VERSION = "0.37.5"
+VERSION = "0.38.0"
 APP_NAME = "cherrytree"
 NEWER_VERSION_URL = "http://www.giuspen.com/software/version_cherrytree"
 if sys.platform.startswith("win"):
@@ -135,9 +136,12 @@ WHITE_SPACE_BETW_PIXB_AND_TEXT = 3
 GRID_SLIP_OFFSET = 3
 MAIN_WIN_TO_TEXT_WIN_NORMALIZER = 50
 MAX_RECENT_DOCS = 10
+MAX_TOOLTIP_LINK_CHARS = 150
 TABLE_DEFAULT_COL_MIN = 40
 TABLE_DEFAULT_COL_MAX = 400
 SCROLL_MARGIN = 0.3
+TREE_DRAG_EDGE_PROX = 10
+TREE_DRAG_EDGE_SCROLL = 15
 
 CHAR_SPACE = " "
 CHAR_NEWLINE = "\n"
@@ -247,9 +251,10 @@ CHERRY_YELLOW, 'image_insert', 'image_edit', 'image_save',
 'insert', 'link_handle', 'link_website', 'cherry_edit',
 'list_bulleted', 'list_numbered', 'list_todo', 'node_name_header',
 'case_toggle', 'case_upper', 'case_lower', 'edit-delete',
-'edit-copy', 'edit-cut', 'edit-paste', 'find',
-'find_again', 'find_back', 'find_replace', 'color_background',
-'color_foreground', 'format-text-large', 'format-text-large2', 'format-text-large3',
+'edit-copy', 'edit-cut', 'edit-paste',
+'find', 'find_sel', 'find_selnsub', 'find_all','find_again', 'find_back', 
+'replace_sel', 'replace_selnsub', 'replace_all', 'find_replace', 'replace_again',
+'color_background', 'color_foreground', 'format-text-large', 'format-text-large2', 'format-text-large3',
 'format-text-small', 'format-text-subscript', 'format-text-superscript', 'format-text-monospace',
 'format-text-strikethrough', 'format-text-underline', 'format-text-bold', 'format-text-italic',
 'format_text_latest', 'format_text_clear', 'format_text',
@@ -262,7 +267,8 @@ CHERRY_YELLOW, 'image_insert', 'image_edit', 'image_save',
 'add', 'cancel', 'done', 'java', 'mail', 'notes', 'python', 'remove', 'star',
 'terminal', 'terminal-red', 'warning', 'home', 'code', 'html',
 'circle-green', 'circle-grey', 'circle-red', 'circle-yellow',
-'locked', 'unlocked', 'people', 'urgent', 'folder', 'leaf', 'xml', 'c', 'cpp',
+'locked', 'unlocked', 'lockpin', 'people', 'urgent', 'folder', 'leaf',
+'xml', 'c', 'cpp', 'perl',
 ]
 NODES_STOCKS_KEYS = [1,2,3,4,5,6,7,8,9,40,41,42,10,43,11,12,13,14,15,16,44,18,19,20,39,38,21,22,23,24,46,47,48,17,25,26,45,27,28,29,30,31,32,33,34,35,36,37]
 NODES_STOCKS = {
@@ -331,7 +337,10 @@ NODES_ICONS = {
 }
 CODE_ICONS = {
 "python": 'python',
+"python3": 'python',
+"perl": 'perl',
 "sh": 'terminal',
+"dosbatch": 'terminal-red',
 "powershell": 'terminal-red',
 "java": 'java',
 "html": 'html',
@@ -354,7 +363,9 @@ is_richtxt INTEGER,
 has_codebox INTEGER,
 has_table INTEGER,
 has_image INTEGER,
-level INTEGER
+level INTEGER,
+ts_creation INTEGER,
+ts_lastsave INTEGER
 )"""
 
 TABLE_CODEBOX_CREATE = """CREATE TABLE codebox (
