@@ -2,7 +2,7 @@
 #
 #       ctdb.py
 #
-#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2017 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #       MA 02110-1301, USA.
 
 import gtk
-import os, sqlite3, xml.dom.minidom, re
+import os, sqlite3, xml.dom.minidom, re, time
 import cons, machines, support, exports
 
 
@@ -34,6 +34,8 @@ class CTDBHandler:
         self.nodes_to_rm_set = set()
         self.bookmarks_to_write = False
         self.remove_at_quit_set = set()
+        self.is_vacuum = False
+        self.db_tables_check_done = False
 
     def reset(self):
         """Reset Variables"""
@@ -65,7 +67,10 @@ class CTDBHandler:
             need_to_commit = True
         self.nodes_to_rm_set.clear()
         if need_to_commit: db.commit()
-        else: print "Writing DB Data but No Updates Found"
+        elif not self.is_vacuum: print "Writing DB Data but No Updates Found"
+        if self.is_vacuum:
+            print "vacuum"
+            db.execute('VACUUM')
 
     def get_image_db_tuple(self, image_element, node_id):
         """From image element to db tuple"""
@@ -249,6 +254,22 @@ class CTDBHandler:
         for child_row in children_rows:
             self.remove_db_node_n_children(db, child_row['node_id'])
 
+    def db_tables_execute_check(self, db):
+        curr_cols_num_node = len(db.execute('PRAGMA table_info(node)').fetchall())
+        if curr_cols_num_node == 11:
+            db.execute('ALTER TABLE node ADD COLUMN ts_creation INTEGER')
+            db.execute('ALTER TABLE node ADD COLUMN ts_lastsave INTEGER')
+            print "table 'node' alter from 11"
+        curr_cols_num_image = len(db.execute('PRAGMA table_info(image)').fetchall())
+        if curr_cols_num_image == 5:
+            db.execute('ALTER TABLE image ADD COLUMN filename TEXT')
+            db.execute('ALTER TABLE image ADD COLUMN link TEXT')
+            db.execute('ALTER TABLE image ADD COLUMN time INTEGER')
+            print "table 'image' alter from 5"
+        elif curr_cols_num_image == 7:
+            db.execute('ALTER TABLE image ADD COLUMN time INTEGER')
+            print "table 'image' alter from 7"
+
     def write_db_node(self, db, tree_iter, level, sequence, node_father_id, write_dict, exporting="", sel_range=None):
         """Write a node in DB"""
         node_id = self.dad.treestore[tree_iter][3]
@@ -268,6 +289,11 @@ class CTDBHandler:
         if foreground:
             is_richtxt |= 0x04
             is_richtxt |= exports.rgb_int24bit_from_str(foreground[1:]) << 3
+        ts_creation = self.dad.treestore[tree_iter][12]
+        ts_lastsave = self.dad.treestore[tree_iter][13]
+        if not self.db_tables_check_done:
+            self.db_tables_execute_check(db)
+            self.db_tables_check_done = True
         if write_dict['buff']:
             if not self.dad.treestore[tree_iter][2]:
                 # we are using db storage and the buffer was not created yet
@@ -280,7 +306,7 @@ class CTDBHandler:
             else:
                 start_iter = curr_buffer.get_iter_at_offset(sel_range[0])
                 end_iter = curr_buffer.get_iter_at_offset(sel_range[1])
-            if is_richtxt:
+            if (is_richtxt & 0x01):
                 # prepare xml dom node
                 if "dom" in dir(self): del self.dom
                 self.dom = xml.dom.minidom.Document()
@@ -328,13 +354,6 @@ class CTDBHandler:
                 else: has_table = 0
                 if images_tuples:
                     has_image = 1
-                    curr_num_cols = len(db.execute('PRAGMA table_info(image)').fetchall())
-                    if curr_num_cols == 5:
-                        db.execute('ALTER TABLE image ADD COLUMN filename TEXT')
-                        db.execute('ALTER TABLE image ADD COLUMN link TEXT')
-                        db.execute('ALTER TABLE image ADD COLUMN time INTEGER')
-                    elif curr_num_cols == 7:
-                        db.execute('ALTER TABLE image ADD COLUMN time INTEGER')
                     db.executemany('INSERT INTO image VALUES(?,?,?,?,?,?,?,?)', images_tuples)
                 else: has_image = 0
                 # retrieve xml text
@@ -350,10 +369,10 @@ class CTDBHandler:
                 db.execute('DELETE FROM node WHERE node_id=?', (node_id,))
             node_tuple = (node_id, name, txt, syntax, tags,
                           is_ro, is_richtxt, has_codebox, has_table, has_image,
-                          level)
-            db.execute('INSERT INTO node VALUES(?,?,?,?,?,?,?,?,?,?,?)', node_tuple)
+                          level, ts_creation, ts_lastsave)
+            db.execute('INSERT INTO node VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?)', node_tuple)
         elif write_dict['buff']:
-            db.execute('UPDATE node SET txt=?, syntax=?, is_richtxt=?, has_codebox=?, has_table=?, has_image=?, level=? WHERE node_id=?', (txt, syntax, is_richtxt, has_codebox, has_table, has_image, level, node_id))
+            db.execute('UPDATE node SET txt=?, syntax=?, is_richtxt=?, has_codebox=?, has_table=?, has_image=?, level=?, ts_lastsave=? WHERE node_id=?', (txt, syntax, is_richtxt, has_codebox, has_table, has_image, level, ts_lastsave, node_id))
         elif write_dict['prop']:
             db.execute('UPDATE node SET name=?, syntax=?, tags=?, is_ro=?, is_richtxt=?, level=? WHERE node_id=?', (name, syntax, tags, is_ro, is_richtxt, level, node_id))
         if write_dict['hier']:
@@ -544,7 +563,10 @@ class CTDBHandler:
 
     def get_node_row_partial_from_id(self, db, node_id):
         """Returns the (partial) node row given the node_id"""
-        node_row = db.execute('SELECT node_id, name, syntax, tags, is_ro, is_richtxt, level FROM node WHERE node_id=?', (node_id,)).fetchone()
+        try:
+            node_row = db.execute('SELECT node_id, name, syntax, tags, is_ro, is_richtxt, level, ts_creation, ts_lastsave FROM node WHERE node_id=?', (node_id,)).fetchone()
+        except:
+            node_row = db.execute('SELECT node_id, name, syntax, tags, is_ro, is_richtxt, level FROM node WHERE node_id=?', (node_id,)).fetchone()
         return node_row
 
     def read_db_node_n_children(self, db, node_row, tree_father, discard_ids, node_sequence):
@@ -574,6 +596,14 @@ class CTDBHandler:
                 syntax_highlighting = cons.RICH_TEXT_ID
         node_level = self.dad.treestore.iter_depth(tree_father)+1 if tree_father else 0
         cherry = self.dad.get_node_icon(node_level, syntax_highlighting, custom_icon_id)
+        try:
+            ts_creation = node_row['ts_creation']
+            ts_lastsave = node_row['ts_lastsave']
+        except IndexError:
+            ts_creation = None
+            ts_lastsave = None
+        if not ts_creation: ts_creation = 0
+        if not ts_lastsave: ts_lastsave = 0
         #print "added node with unique_id", unique_id
         # insert the node containing the buffer into the tree
         tree_iter = self.dad.treestore.append(tree_father, [cherry,
@@ -587,7 +617,10 @@ class CTDBHandler:
                                                             None,
                                                             custom_icon_id,
                                                             support.get_pango_weight(is_bold),
-                                                            foreground])
+                                                            foreground,
+                                                            ts_creation,
+                                                            ts_lastsave])
+        self.dad.update_node_aux_icon(tree_iter)
         self.dad.nodes_names_dict[self.dad.treestore[tree_iter][3]] = self.dad.treestore[tree_iter][1]
         if discard_ids:
             # we are importing (=> adding) a node
@@ -608,6 +641,12 @@ class CTDBHandler:
 
     def read_db_full(self, db, discard_ids, tree_father=None):
         """Read the whole DB"""
+        self.db_tables_check_done = False
+        # bookmarks
+        bookmarks_rows = db.execute('SELECT * FROM bookmark ORDER BY sequence ASC').fetchall()
+        for bookmark_row in bookmarks_rows:
+            node_id = bookmark_row['node_id']
+            self.dad.bookmarks.append(str(node_id))
         # tree nodes
         node_sequence = 0
         children_rows = self.get_children_rows_from_father_id(db, 0)
@@ -620,12 +659,4 @@ class CTDBHandler:
                                              tree_father,
                                              discard_ids,
                                              node_sequence)
-        # bookmarks
-        bookmarks_rows = db.execute('SELECT * FROM bookmark ORDER BY sequence ASC').fetchall()
-        for bookmark_row in bookmarks_rows:
-            node_id = bookmark_row['node_id']
-            self.dad.bookmarks.append(str(node_id))
-            tree_iter = self.dad.get_tree_iter_from_node_id(node_id)
-            if tree_iter:
-                self.dad.update_node_pre_icon(tree_iter, stock_id="pin")
         self.dad.nodes_sequences_fix(tree_father, False)
