@@ -2,7 +2,7 @@
 #
 #       machines.py
 #
-#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2017 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -19,8 +19,16 @@
 #       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #       MA 02110-1301, USA.
 
-import gtk, xml.dom.minidom, re, base64, copy, StringIO
-import cons, config, support, exports
+import gtk
+import xml.dom.minidom
+import re
+import base64
+import copy
+import StringIO
+import cons
+import config
+import support
+import exports
 
 
 def get_blob_buffer_from_pixbuf(pixbuf):
@@ -109,10 +117,6 @@ class XMLHandler:
                     self.append_tree_node(dom_iter, tree_father, discard_ids, node_sequence)
                 elif dom_iter.nodeName == "bookmarks":
                     self.dad.bookmarks = dom_iter.attributes['list'].value.split(",")
-                    for node_id_str in self.dad.bookmarks:
-                        tree_iter = self.dad.get_tree_iter_from_node_id(int(node_id_str))
-                        if tree_iter:
-                            self.dad.update_node_pre_icon(tree_iter, stock_id="pin")
                 dom_iter = dom_iter.nextSibling
             return True
 
@@ -136,6 +140,8 @@ class XMLHandler:
                 syntax_highlighting = cons.RICH_TEXT_ID
         node_depth = 0 if not tree_father else self.dad.treestore.iter_depth(tree_father)+1
         cherry = self.dad.get_node_icon(node_depth, syntax_highlighting, custom_icon_id)
+        ts_creation = float(dom_iter.attributes['ts_creation'].value) if dom_iter.hasAttribute('ts_creation') else 0
+        ts_lastsave = float(dom_iter.attributes['ts_lastsave'].value) if dom_iter.hasAttribute('ts_lastsave') else 0
         curr_buffer = self.dad.buffer_create(syntax_highlighting)
         if syntax_highlighting != cons.RICH_TEXT_ID: curr_buffer.begin_not_undoable_action()
         # loop into rich text, write into the buffer
@@ -164,7 +170,10 @@ class XMLHandler:
                                                             None,
                                                             custom_icon_id,
                                                             support.get_pango_weight(is_bold),
-                                                            foreground])
+                                                            foreground,
+                                                            ts_creation,
+                                                            ts_lastsave])
+        self.dad.update_node_aux_icon(tree_iter)
         self.dad.nodes_names_dict[unique_id] = self.dad.treestore[tree_iter][1]
         if discard_ids:
             # we are importing nodes
@@ -294,12 +303,13 @@ class XMLHandler:
         self.dom = xml.dom.minidom.Document()
         cherrytree = self.dom.createElement(cons.APP_NAME)
         self.dom.appendChild(cherrytree)
+        # bookmarks first
+        self.append_bookmarks(cherrytree)
         # full tree parsing
         tree_iter = self.dad.treestore.get_iter_first()
         while tree_iter != None:
             self.append_dom_node(tree_iter, cherrytree, to_disk=True)
             tree_iter = self.dad.treestore.iter_next(tree_iter)
-        self.append_bookmarks(cherrytree)
         return self.dom.toprettyxml()
 
     def append_bookmarks(self, dom_father):
@@ -320,6 +330,8 @@ class XMLHandler:
         dom_iter.setAttribute("readonly", str(self.dad.treestore[tree_iter][7]))
         dom_iter.setAttribute("custom_icon_id", str(self.dad.treestore[tree_iter][9]))
         dom_iter.setAttribute("is_bold", str(support.get_pango_is_bold(self.dad.treestore[tree_iter][10])))
+        dom_iter.setAttribute("ts_creation", str(self.dad.treestore[tree_iter][12]))
+        dom_iter.setAttribute("ts_lastsave", str(self.dad.treestore[tree_iter][13]))
         foreground = self.dad.treestore[tree_iter][11]
         if not foreground: foreground = ""
         dom_iter.setAttribute("foreground", foreground)
@@ -679,6 +691,7 @@ class StateMachine:
 
     def __init__(self, dad):
         """Machine boot"""
+        self.dad = dad
         self.nodes_vectors = {}
         self.nodes_indexes = {}
         self.nodes_indicators = {}
@@ -687,7 +700,7 @@ class StateMachine:
         # indicator 0 -> current text in node_vector matches the textbuffer
         # indicator 1 -> textbuffer is ahead, but only of non alphanumeric chars
         # indicator 2 -> textbuffer is ahead with alphanumeric chars
-        self.dad = dad
+        self.not_undoable_timeslot_set(False)
 
     def get_embedded_pixbufs_tables_codeboxes(self, text_buffer, for_print=0, sel_range=None):
         """Retrieve the list of Images Embedded into the Buffer"""
@@ -887,9 +900,15 @@ class StateMachine:
         last_index = len(self.nodes_vectors[node_id]) - 1
         return curr_index == last_index
 
+    def not_undoable_timeslot_set(self, not_undoable_val):
+        self._not_undoable_timeslot = not_undoable_val
+
+    def not_undoable_timeslot_get(self):
+        return self._not_undoable_timeslot
+
     def update_state(self):
         """Update the state for the given node_id"""
-        if self.dad.syntax_highlighting != cons.RICH_TEXT_ID:
+        if self.dad.syntax_highlighting != cons.RICH_TEXT_ID or self.not_undoable_timeslot_get():
             return
         node_id = self.dad.treestore[self.dad.curr_tree_iter][3]
         if not self.curr_index_is_last_index(node_id):
