@@ -2,7 +2,7 @@
 #
 #       support.py
 #
-#       Copyright 2009-2016 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2017 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -19,10 +19,35 @@
 #       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #       MA 02110-1301, USA.
 
-import gtk, pango, locale, os, webbrowser, re
-import cons, config, exports
+import gtk
+import pango
+import locale
+import os
+import webbrowser
+import re
+import time
+import cons
+import config
+import exports
 
 
+def get_timestamp_str(timestamp_format, time_float):
+    """Get timestamp printable from float"""
+    struct_time = time.localtime(time_float)
+    try:
+        timestamp_str = time.strftime(timestamp_format, struct_time).decode(locale.getlocale()[1])
+    except:
+        timestamp_str = time.strftime(config.TIMESTAMP_FORMAT_DEFAULT, struct_time).decode(locale.getlocale()[1])
+    return timestamp_str
+
+def get_word_count(dad):
+    if dad.curr_buffer:
+        all_text = unicode(dad.curr_buffer.get_text(*dad.curr_buffer.get_bounds()), cons.STR_UTF8, cons.STR_IGNORE)
+        word_count = len([w for w in all_text.split() if re.search("\w", w, re.UNICODE)])
+    else:
+        word_count = 0
+    return word_count
+
 def auto_decode_str(in_str, from_clipboard=False):
     """Try to Detect Encoding and Decode"""
     try:
@@ -53,33 +78,68 @@ def auto_decode_str(in_str, from_clipboard=False):
         out_str = unicode(in_str, cons.STR_UTF8, cons.STR_IGNORE)
     return out_str
 
-def apply_tag_try_link(dad, iter_end, iter_cursor=None):
-    """Try and apply link to previous word (after space or newline)"""
-    iter_start = iter_end.copy()
-    while iter_start.backward_char():
-        curr_char = iter_start.get_char()
-        if curr_char in cons.WEB_LINK_SEPARATORS:
-            iter_start.forward_char()
-            break
-    num_chars = iter_end.get_offset() - iter_start.get_offset()
-    tag_applied = False
-    if num_chars > 4 and get_next_chars_from_iter_are(iter_start, cons.WEB_LINK_STARTERS):
+def apply_tag_try_node_name(dad, iter_start, iter_end):
+    """Apply Link to Node Tag if the text is a node name"""
+    node_name = dad.curr_buffer.get_text(iter_start, iter_end)
+    node_dest = dad.get_tree_iter_from_node_name(node_name)
+    if node_dest:
         dad.curr_buffer.select_range(iter_start, iter_end)
-        link_url = dad.curr_buffer.get_text(iter_start, iter_end)
-        if link_url[0:3] not in ["htt", "ftp"]: link_url = "http://" + link_url
-        property_value = cons.LINK_TYPE_WEBS + cons.CHAR_SPACE + link_url
+        property_value = cons.LINK_TYPE_NODE + cons.CHAR_SPACE + str(dad.treestore[node_dest][3])
         dad.apply_tag(cons.TAG_LINK, property_value=property_value)
-        tag_applied = True
-    elif num_chars > 2 and get_is_camel_case(iter_start, num_chars):
-        node_name = dad.curr_buffer.get_text(iter_start, iter_end)
-        node_dest = dad.get_tree_iter_from_node_name(node_name)
-        if node_dest:
+        return True
+    return False
+
+def apply_tag_try_link(dad, iter_end, offset_cursor=None):
+    """Try and apply link to previous word (after space or newline)"""
+    tag_applied = False
+    iter_start = iter_end.copy()
+    if iter_start.backward_char() and iter_start.get_char() == cons.CHAR_SQ_BR_CLOSE\
+    and iter_start.backward_char() and iter_start.get_char() == cons.CHAR_SQ_BR_CLOSE:
+        curr_state = 0
+        end_offset = iter_start.get_offset()
+        while iter_start.backward_char():
+            curr_char = iter_start.get_char()
+            if curr_char == cons.CHAR_NEWLINE:
+                break
+            if curr_char == cons.CHAR_SQ_BR_OPEN:
+                if curr_state == 0:
+                    curr_state = 1
+                else:
+                    curr_state = 2
+                    break
+        if curr_state == 2:
+            start_offset = iter_start.get_offset()+2
+            end_offset = iter_end.get_offset()-2
+            if apply_tag_try_node_name(dad, dad.curr_buffer.get_iter_at_offset(start_offset),
+                                       dad.curr_buffer.get_iter_at_offset(end_offset)):
+                tag_applied = True
+                dad.curr_buffer.delete(dad.curr_buffer.get_iter_at_offset(end_offset),
+                                       dad.curr_buffer.get_iter_at_offset(end_offset+2))
+                dad.curr_buffer.delete(dad.curr_buffer.get_iter_at_offset(start_offset-2),
+                                       dad.curr_buffer.get_iter_at_offset(start_offset))
+                if offset_cursor != None:
+                    offset_cursor -= 4
+    else:
+        iter_start = iter_end.copy()
+        while iter_start.backward_char():
+            curr_char = iter_start.get_char()
+            if curr_char in cons.WEB_LINK_SEPARATORS:
+                iter_start.forward_char()
+                break
+        num_chars = iter_end.get_offset() - iter_start.get_offset()
+        if num_chars > 4 and get_next_chars_from_iter_are(iter_start, cons.WEB_LINK_STARTERS):
             dad.curr_buffer.select_range(iter_start, iter_end)
-            property_value = cons.LINK_TYPE_NODE + cons.CHAR_SPACE + str(dad.treestore[node_dest][3])
+            link_url = dad.curr_buffer.get_text(iter_start, iter_end)
+            if link_url[0:3] not in ["htt", "ftp"]: link_url = "http://" + link_url
+            property_value = cons.LINK_TYPE_WEBS + cons.CHAR_SPACE + link_url
             dad.apply_tag(cons.TAG_LINK, property_value=property_value)
             tag_applied = True
-    if tag_applied and iter_cursor:
-        dad.curr_buffer.place_cursor(iter_cursor)
+        elif num_chars > 2 and get_is_camel_case(iter_start, num_chars):
+            if apply_tag_try_node_name(dad, iter_start, iter_end):
+                tag_applied = True
+    if tag_applied and offset_cursor != None:
+        dad.curr_buffer.place_cursor(dad.curr_buffer.get_iter_at_offset(offset_cursor))
+    return tag_applied
 
 def apply_tag_try_automatic_bounds(dad, text_buffer=None, iter_start=None):
     """Try to Select a Word Forward/Backward the Cursor"""
@@ -218,10 +278,13 @@ def on_sourceview_event_after_scroll(dad, text_view, event):
 
 def on_sourceview_event_after_key_release(dad, text_view, event):
     """Called after every gtk.gdk.KEY_RELEASE on the SourceView"""
+    keyname = gtk.gdk.keyval_name(event.keyval)
     if dad.ctrl_down:
-        keyname = gtk.gdk.keyval_name(event.keyval)
         if keyname in cons.STR_KEYS_CONTROL:
             dad.ctrl_down = False
+    elif keyname in [cons.STR_KEY_RETURN, cons.STR_KEY_SPACE]:
+        if dad.word_count:
+            dad.update_selected_node_statusbar_info()
     return False
 
 def on_sourceview_event_after_key_press(dad, text_view, event, syntax_highl):
@@ -274,7 +337,8 @@ def on_sourceview_event_after_key_press(dad, text_view, event, syntax_highl):
         if syntax_highl == cons.RICH_TEXT_ID:
             iter_end_link = iter_insert.copy()
             iter_end_link.backward_char()
-            apply_tag_try_link(dad, iter_end_link, iter_insert)
+            if apply_tag_try_link(dad, iter_end_link, iter_insert.get_offset()):
+                iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
         iter_start = iter_insert.copy()
         if keyname == cons.STR_KEY_RETURN:
             cursor_key_press = iter_insert.get_offset()
@@ -432,6 +496,8 @@ def sourceview_cursor_and_tooltips_handler(dad, text_view, x, y):
         #print "link", dad.hovering_link_iter_offset
     if dad.hovering_link_iter_offset >= 0:
         text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
+        if len(tooltip) > cons.MAX_TOOLTIP_LINK_CHARS:
+            tooltip = tooltip[:cons.MAX_TOOLTIP_LINK_CHARS] + "..."
         text_view.set_tooltip_text(tooltip)
     else:
         text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
@@ -769,7 +835,7 @@ def dialog_about(dad):
     dialog = gtk.AboutDialog()
     dialog.set_program_name("CherryTree")
     dialog.set_version(cons.VERSION)
-    dialog.set_copyright("""Copyright © 2009-2016
+    dialog.set_copyright("""Copyright © 2009-2017
 Giuseppe Penone <giuspen@gmail.com>""")
     dialog.set_comments(_("A Hierarchical Note Taking Application, featuring Rich Text and Syntax Highlighting"))
     dialog.set_license(_("""
@@ -795,7 +861,7 @@ MA 02110-1301, USA.
     dialog.set_translator_credits(_("Armenian")+" (hy) Seda Stamboltsyan <sedastam@yandex.com>"+cons.CHAR_NEWLINE+
 _("Chinese Simplified")+" (zh_CN) Channing Wong <channing.wong@qq.com>"+cons.CHAR_NEWLINE+
 _("Czech")+" (cs) Pavel Fric <fripohled@blogspot.com>"+cons.CHAR_NEWLINE+
-_("Dutch")+" (nl) Patrick Vijgeboom <pj.vijgeboom@gmail.com>"+cons.CHAR_NEWLINE+
+_("Dutch")+" (nl) Luuk Geurts"+cons.CHAR_NEWLINE+
 _("French")+" (fr) Klaus Becker <colonius@free.fr>"+cons.CHAR_NEWLINE+
 _("German")+" (de) Frank Brungräber <calexu@arcor.de>"+cons.CHAR_NEWLINE+
 _("Greek")+" (el) Delphina <delphina.2009@yahoo.gr>"+cons.CHAR_NEWLINE+
@@ -1304,17 +1370,17 @@ def dialog_link_handle(dad, title, sel_tree_iter):
         if radiobutton.get_active(): dad.link_type = cons.LINK_TYPE_FOLD
         link_type_changed_on_dialog()
     def on_button_browse_for_file_to_link_to_clicked(self, *args):
-        filepath = dialog_file_select(curr_folder=dad.pick_dir, parent=dialog)
+        filepath = dialog_file_select(curr_folder=dad.pick_dir_file, parent=dialog)
         if not filepath: return
-        dad.pick_dir = os.path.dirname(filepath)
+        dad.pick_dir_file = os.path.dirname(filepath)
         if dad.links_relative:
             try: filepath = os.path.relpath(filepath, dad.file_dir)
             except: print "cannot set relative path for different drives"
         entry_file.set_text(filepath)
     def on_button_browse_for_folder_to_link_to_clicked(self, *args):
-        filepath = dialog_folder_select(curr_folder=dad.pick_dir, parent=dialog)
+        filepath = dialog_folder_select(curr_folder=dad.pick_dir_file, parent=dialog)
         if not filepath: return
-        dad.pick_dir = filepath
+        dad.pick_dir_file = filepath
         if dad.links_relative:
             try: filepath = os.path.relpath(filepath, dad.file_dir)
             except: print "cannot set relative path for different drives"
@@ -1398,6 +1464,106 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     dad.links_entries['node'] = links_parms.sel_iter
     return True
 
+def dialog_choose_data_storage(dad):
+    """Choose the CherryTree data storage type (xml or db) and protection"""
+    dialog = gtk.Dialog(title=_("Choose Storage Type"),
+                        parent=dad.window,
+                        flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
+                        buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
+                        gtk.STOCK_OK, gtk.RESPONSE_ACCEPT) )
+    dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
+    dialog.set_default_size(350, -1)
+    radiobutton_sqlite_not_protected = gtk.RadioButton(label="SQLite, " + _("Not Protected") + " (.ctb)")
+    radiobutton_sqlite_pass_protected = gtk.RadioButton(label="SQLite, " + _("Password Protected") + " (.ctx)")
+    radiobutton_sqlite_pass_protected.set_group(radiobutton_sqlite_not_protected)
+    radiobutton_xml_not_protected = gtk.RadioButton(label="XML, " + _("Not Protected") + " (.ctd)")
+    radiobutton_xml_not_protected.set_group(radiobutton_sqlite_not_protected)
+    radiobutton_xml_pass_protected = gtk.RadioButton(label="XML, " + _("Password Protected") + " (.ctz)")
+    radiobutton_xml_pass_protected.set_group(radiobutton_sqlite_not_protected)
+    type_vbox = gtk.VBox()
+    type_vbox.pack_start(radiobutton_sqlite_not_protected)
+    type_vbox.pack_start(radiobutton_sqlite_pass_protected)
+    type_vbox.pack_start(radiobutton_xml_not_protected)
+    type_vbox.pack_start(radiobutton_xml_pass_protected)
+    type_frame = gtk.Frame(label="<b>"+_("Storage Type")+"</b>")
+    type_frame.get_label_widget().set_use_markup(True)
+    type_frame.set_shadow_type(gtk.SHADOW_NONE)
+    type_frame.add(type_vbox)
+    entry_passw_1 = gtk.Entry()
+    entry_passw_1.set_visibility(False)
+    entry_passw_2 = gtk.Entry()
+    entry_passw_2.set_visibility(False)
+    label_passwd = gtk.Label(_("CT saves the document in an encrypted 7zip archive. When viewing or editing the document, CT extracts the encrypted archive to a temporary folder, and works on the unencrypted copy. When closing, the unencrypted copy is deleted from the temporary directory. Note that in the case of application or system crash, the unencrypted document will remain in the temporary folder."))
+    label_passwd.set_width_chars(70)
+    label_passwd.set_line_wrap(True)
+    vbox_passw = gtk.VBox()
+    vbox_passw.pack_start(entry_passw_1)
+    vbox_passw.pack_start(entry_passw_2)
+    vbox_passw.pack_start(label_passwd)
+    passw_frame = gtk.Frame(label="<b>"+_("Enter the New Password Twice")+"</b>")
+    passw_frame.get_label_widget().set_use_markup(True)
+    passw_frame.set_shadow_type(gtk.SHADOW_NONE)
+    passw_frame.add(vbox_passw)
+    if len(dad.file_name) > 4:
+        if dad.file_name[-1] == "b": radiobutton_sqlite_not_protected.set_active(True)
+        elif dad.file_name[-1] == "x": radiobutton_sqlite_pass_protected.set_active(True)
+        elif dad.file_name[-1] == "d": radiobutton_xml_not_protected.set_active(True)
+        else: radiobutton_xml_pass_protected.set_active(True)
+    if dad.password: passw_frame.set_sensitive(True)
+    else: passw_frame.set_sensitive(False)
+    content_area = dialog.get_content_area()
+    content_area.set_spacing(5)
+    content_area.pack_start(type_frame)
+    content_area.pack_start(passw_frame)
+    content_area.show_all()
+    def on_radiobutton_savetype_toggled(widget):
+        if radiobutton_sqlite_pass_protected.get_active()\
+        or radiobutton_xml_pass_protected.get_active():
+            passw_frame.set_sensitive(True)
+            entry_passw_1.grab_focus()
+        else: passw_frame.set_sensitive(False)
+    def on_key_press_edit_data_storage_type_dialog(widget, event):
+        if gtk.gdk.keyval_name(event.keyval) == cons.STR_KEY_RETURN:
+            try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
+            except: print cons.STR_PYGTK_222_REQUIRED
+            return True
+        return False
+    radiobutton_sqlite_not_protected.connect("toggled", on_radiobutton_savetype_toggled)
+    radiobutton_sqlite_pass_protected.connect("toggled", on_radiobutton_savetype_toggled)
+    radiobutton_xml_not_protected.connect("toggled", on_radiobutton_savetype_toggled)
+    dialog.connect("key_press_event", on_key_press_edit_data_storage_type_dialog)
+    response = dialog.run()
+    storage_type_is_xml = (radiobutton_xml_not_protected.get_active()\
+                           or radiobutton_xml_pass_protected.get_active())
+    new_protection = {'on': (radiobutton_sqlite_pass_protected.get_active()\
+                             or radiobutton_xml_pass_protected.get_active()),
+                      'p1': unicode(entry_passw_1.get_text(), cons.STR_UTF8, cons.STR_IGNORE),
+                      'p2': unicode(entry_passw_2.get_text(), cons.STR_UTF8, cons.STR_IGNORE)}
+    dialog.destroy()
+    if response != gtk.RESPONSE_ACCEPT: return False
+    if new_protection['on']:
+        if new_protection['p1'] == "":
+            dialog_error(_("The Password Fields Must be Filled"), dad.window)
+            return False
+        if new_protection['p1'] != new_protection['p2']:
+            dialog_error(_("The Two Inserted Passwords Do Not Match"), dad.window)
+            return False
+        for bad_char in cons.CHARS_NOT_FOR_PASSWD:
+            if bad_char in new_protection['p1']:
+                dialog_error(_("The Characters  %s  are Not Allowed") % cons.CHAR_SPACE.join(cons.CHARS_NOT_FOR_PASSWD), dad.window)
+                return False
+        if not new_protection['p1'] or not dad.is_7za_available(): return False
+        dad.password = new_protection['p1']
+    else: dad.password = None
+    if storage_type_is_xml:
+        if dad.password: dad.filetype = "z"
+        else: dad.filetype = "d"
+    else:
+        if dad.password: dad.filetype = "x"
+        else: dad.filetype = "b"
+    #print "dad.filetype = '%s'" % dad.filetype
+    return True
+
 def dialog_choose_node(dad, title, treestore, sel_tree_iter):
     """Dialog to Select a Node"""
     class NodeParms:
@@ -1545,6 +1711,18 @@ def dialog_color_pick(dad, curr_color=None):
     colorselection.set_has_palette(True)
     if curr_color:
         colorselection.set_current_color(curr_color)
+    def on_key_press_color_pick_dialog(widget, event):
+        if gtk.gdk.keyval_name(event.keyval) == cons.STR_KEY_RETURN:
+            try: dialog.get_widget_for_response(gtk.RESPONSE_OK).clicked()
+            except: print cons.STR_PYGTK_222_REQUIRED
+            return True
+        return False
+    dialog.connect("key_press_event", on_key_press_color_pick_dialog)
+    def on_mouse_button_clicked_color_pick_dialog(widget, event):
+        if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
+            try: dialog.get_widget_for_response(gtk.RESPONSE_OK).clicked()
+            except: print cons.STR_PYGTK_222_REQUIRED
+    dialog.connect('button-press-event', on_mouse_button_clicked_color_pick_dialog)
     response = dialog.run()
     dialog.hide()
     if response != gtk.RESPONSE_OK: return None
@@ -1653,6 +1831,7 @@ def insert_special_char(menu_item, special_char, dad):
     """A Special character insert was Requested"""
     text_view, text_buffer, from_codebox = dad.get_text_view_n_buffer_codebox_proof()
     if not text_buffer: return
+    if not dad.is_curr_node_not_read_only_or_error(): return
     text_buffer.insert_at_cursor(special_char)
 
 def open_recent_document(menu_item, filepath, dad):
@@ -1672,7 +1851,10 @@ def open_recent_document(menu_item, filepath, dad):
 def select_bookmark_node(menu_item, node_id_str, dad):
     """Select a Node in the Bookmarks List"""
     node_iter = dad.get_tree_iter_from_node_id(long(node_id_str))
-    if node_iter: dad.treeview_safe_set_cursor(node_iter)
+    if node_iter:
+        dad.treeview_safe_set_cursor(node_iter)
+        if dad.tree_click_expand:
+            dad.treeview.expand_row(dad.treestore.get_path(node_iter), open_all=False)
 
 def bookmarks_handle(dad):
     """Handle the Bookmarks List"""
@@ -1786,7 +1968,7 @@ def bookmarks_handle(dad):
     for removed_bookmark in removed_bookmarks:
         tree_iter = dad.get_tree_iter_from_node_id(int(removed_bookmark))
         if tree_iter:
-            dad.update_node_pre_icon(tree_iter)
+            dad.update_node_aux_icon(tree_iter)
             if dad.curr_tree_iter and dad.treestore[tree_iter][3] == dad.treestore[dad.curr_tree_iter][3]:
                 dad.menu_tree_update_for_bookmarked_node(False)
     set_bookmarks_menu_items(dad)
