@@ -2,7 +2,7 @@
 #
 #       tablez.py
 #
-#       Copyright 2009-2015 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2017 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 
 import gtk, pango
 import os, csv, codecs, cStringIO, copy
-import cons, support
+import cons, menus, support
 
 
 class TablesHandler:
@@ -265,13 +265,13 @@ class TablesHandler:
 
     def table_export(self, action):
         """Table Export as CSV File"""
-        filename = support.dialog_file_save_as(curr_folder=self.dad.pick_dir,
+        filename = support.dialog_file_save_as(curr_folder=self.dad.pick_dir_csv,
                                                filter_pattern="*.csv",
                                                filter_name=_("CSV File"),
                                                parent=self.dad.window)
         if not filename: return
         if len(filename) < 4 or filename[-4:] != ".csv": filename += ".csv"
-        self.dad.pick_dir = os.path.dirname(filename)
+        self.dad.pick_dir_csv = os.path.dirname(filename)
         table_dict = self.dad.state_machine.table_to_dict(self.curr_table_anchor)
         table_matrix = table_dict['matrix']
         table_matrix.insert(0, table_matrix.pop())
@@ -290,10 +290,10 @@ class TablesHandler:
         else:
             filepath = support.dialog_file_select(filter_pattern=["*.csv"],
                 filter_name=_("CSV File"),
-                curr_folder=self.dad.pick_dir,
+                curr_folder=self.dad.pick_dir_csv,
                 parent=self.dad.window)
             if filepath != None:
-                self.dad.pick_dir = os.path.dirname(filepath)
+                self.dad.pick_dir_csv = os.path.dirname(filepath)
                 support.text_file_rm_emptylines(filepath)
                 file_descriptor = open(filepath, 'r')
                 reader = UnicodeReader(file_descriptor)
@@ -389,7 +389,7 @@ class TablesHandler:
     def on_table_cell_populate_popup(self, entry, menu):
         """Table Cell Populate Popup"""
         self.curr_table_cell = entry
-        self.dad.menu_populate_popup(menu, cons.get_popup_menu_entries_table_cell(self))
+        self.dad.menu_populate_popup(menu, menus.get_popup_menu_entries_table_cell(self))
 
     def curr_table_cell_insert_newline(self, *args):
         if not self.dad.is_curr_node_not_read_only_or_error(): return
@@ -621,7 +621,7 @@ class TablesHandler:
             self.curr_table_anchor = anchor
             self.dad.object_set_selection(self.curr_table_anchor)
             menu_table = gtk.Menu()
-            self.dad.menu_populate_popup(menu_table, cons.get_popup_menu_table(self.dad))
+            self.dad.menu_populate_popup(menu_table, menus.get_popup_menu_table(self.dad))
             menu_table.popup(None, None, None, 3, event.time)
             return True
         return False
@@ -632,7 +632,7 @@ class TablesHandler:
         self.dad.object_set_selection(self.curr_table_anchor)
         if event.button == 3:
             menu_table = gtk.Menu()
-            self.dad.menu_populate_popup(menu_table, cons.get_popup_menu_table(self.dad))
+            self.dad.menu_populate_popup(menu_table, menus.get_popup_menu_table(self.dad))
             menu_table.popup(None, None, None, event.button, event.time)
             return True
         return False
